-- ###################################
-- 3D STUDIO MAX TO JOT SCRIPT
-- SALMAN BUTT
-- CS DEPARTMENT, PRINCETON UNIVERSITY
-- CREATED: 07/01/02
-- LAST MODIFIED: 5/28/03
-- VERSION 0.8
-- ###################################

-- ###################################
-- INITALIZE JOT FILE

fname = getSaveFileName caption:"Save jot output file" filename:"scene"
if fname != undefined then f = createFile(fname + ".jot")
file_pfx=getFilenameFile fname
format "#jot\n\n" to:f

-- ###################################
-- INITIALIZE VARIABLES

obj_array =  $* as array
mesh_objs = #()
light_objs = #()
cam_objs=#()
mctr=1
cctr=1
lctr=1
startFrame=(animationrange.start.frame) as integer
endFrame=(animationrange.end.frame) as integer
nframes=endFrame-startFrame+1
xres=renderWidth
yres=renderHeight
ctr=1
fctr=1

-- ###################################
-- INITIALIZE OBJECT ARRAYS

for obj in obj_array do
	(
		result = case superclassof obj of
		(
			GeometryClass:
			(
				append mesh_objs obj
				mctr+=1
			)
			Camera:
			(
				append cam_objs obj
				cctr+=1
			)
			Light:
			(
				append light_objs obj
				lctr+=1
			)
			default:
			(
				-- TAKE NO ACTION FOR DEFAULT OBJECT TYPES
			)
		)
	)
	if mctr==1 then messagebox ("Warning! No meshes found")
	if cctr==1 then messagebox ("Warning! No camera found")
	if cctr>2 then messagebox ("Warning! More than one camera found. Using first one.")

-- ###################################
-- FUNCTIONS

fn leadingZero num len = 
(
	local t = num as string
	while t.count < len do 
		t = "0" + t
	t
)

fn writeMesh mesh_obj filename =
(
	format "\nLMESH \t{ \n" to:filename

	-- WRITE VERTS
	nverts=mesh_obj.numverts
	varray = mesh_obj.verts
	ctr2=1
	format "\tvertices \t{ {" to:filename
	for ctr2=1 to nverts do
	(
		format "{% % % }" (in coordsys local varray[ctr2].pos[1]) (in coordsys local varray[ctr2].pos[3]) (in coordsys local -varray[ctr2].pos[2]) to:filename
	)
	format "}}\n" to:filename

	-- WRITE FACES
	nfaces = mesh_obj.numfaces
	farray = mesh_obj.faces
	format "\tfaces \t{ {" to:filename
	for ctr2=1 to nfaces do
	(
		format "{% % % }" (((getFace mesh_obj ctr2)[1] as integer)-1) (((getFace mesh_obj ctr2)[2] as integer)-1) (((getFace mesh_obj ctr2)[3] as integer)-1) to:filename
	)
	format "}}\n" to:filename

	format "\tpatch \t{ \n" to:filename
	format "\t\tPatch \t{ \n" to:filename
	format "\t\t\tcur_tex \t0 \n" to:filename
	format "\t\t\tpatchname \tpatch-0 \n" to:filename
	format "\t\t\tcolor \t{1 1 1 } \n" to:filename
	format "\t\t\ttexture \t{ \n" to:filename
	format "\t\t\t\tNPRTexture \t{ \n" to:filename
	format "\t\t\t\t\tnpr_data_file \t{ % } \n" mesh_obj.name to:filename
	format "\t\t\t\t\t} } \n" to:filename
	format "\t\t} }\n" to:filename

	format "\t} " to:filename
)

fn writeMeshVerts mesh_obj filename =
(
	format "\nLMESH\t{\n" to:filename

	nverts=mesh_obj.numverts
	varray = mesh_obj.verts
	ctr2=1
	format "\tvertices\t{ {\t" to:filename
	for ctr2=1 to nverts do
	(
		format "{% % %}" (in coordsys local varray[ctr2].pos[1]) (in coordsys local varray[ctr2].pos[3]) (in coordsys local -varray[ctr2].pos[2]) to:filename
	)
	format "\n\t\t}}\n\t}\n" to:filename
)

fn GetCamVFOV cam =
(
	local r_aspect=(renderwidth as float)/renderHeight
	2.0*atan(tan(cam.fov/2.)/r_aspect)
)

fn writeCamera cam filename =
(
	format "CHNG_CAM\t{ " to:filename

	-- FROM:
	format "{% % % }" cam.transform[4][1] cam.transform[4][3] -cam.transform[4][2] to:filename

	-- AT:
	format "{% % % }" (cam.transform[4][1]+cam.transform[3][1]) (cam.transform[4][3]-cam.transform[3][3]) (-cam.transform[4][2]+cam.transform[3][2]) to:filename

	-- UP:
	format "{% % % }" (cam.transform[4][1]+cam.transform[2][1]) (cam.transform[4][3]+cam.transform[2][3]) (-cam.transform[4][2]-cam.transform[2][2]) to:filename

	-- CENTER:
	format "{% % % }" (cam.transform[4][1]-cam.transform[3][1]) (cam.transform[4][3]-cam.transform[3][3]) (-cam.transform[4][2]+cam.transform[3][2]) to:filename

	-- FOCAL LENGTH, 1, IOD (aspect ratio)
	if (renderWidth>=renderHeight) then format "% 1 2.25\n" ((.05/(tan (GetCamVFOV cam/2.))))  to:filename
	else format "% 1 2.25\n" ((.05/(tan (cam.fov/2.))))  to:filename

	format "\t}\n" to:filename
)

fn writeWindow filename =
(
	format "CHNG_WIN\t{ 3 29 % % \n\t}\n" xres yres to:filename
)

fn writeView filename =
(
	format "CHNG_VIEW\t{\n" to:filename
	format "\tVIEW\t{\n" to:filename
	format "\t\tview_animator\t{\n" to:filename
	format "\t\t\tAnimator\t{\n" to:filename
	format "\t\t\t\tfps\t%\n" frameRate to:filename
	format "\t\t\t\tstart_frame\t%\n" (startFrame+1) to:filename
	format "\t\t\t\tend_frame\t%\n" (endFrame+1) to:filename
	format "\t\t\t\tname\t{ % }\n" file_pfx to:filename
	format "\t\t\t\t} }\n" to:filename
	format "\t\tview_data_file\t{ % }\n" file_pfx to:filename
	format "\t\t}\n" to:filename
	format "\t}\n" to:filename
)

fn writeTexbody mesh_obj filename =
(
	format "TEXBODY\t{\n" to:filename
	format "\t name\t%\n" (file_pfx+"-"+mesh_obj.name) to:filename
	format "\t xform\t{" to:filename
	
	format "{% % % % }{% % % % }{% % % % }" \
	mesh_obj.transform[1][1] mesh_obj.transform[3][1] -mesh_obj.transform[2][1] mesh_obj.transform[4][1] \
	mesh_obj.transform[1][3] mesh_obj.transform[3][3] -mesh_obj.transform[2][3] mesh_obj.transform[4][3] \
	-mesh_obj.transform[1][2] -mesh_obj.transform[3][2] mesh_obj.transform[2][2] -mesh_obj.transform[4][2] \
	to:filename

	format "{0.0 0.0 0.0 1.0 }}\n" to:filename
	format "\t xfdef\t{ DEFINER\n" to:filename
	format "\t\tDEFINER\t{\n" to:filename
	format "\t\t\tout_mask\t1\n" to:filename
	format "\t\t\tinputs\t{ }\n" to:filename
	format "\t\t\t} }\n" to:filename
	
	-- COLOR - HACKED TO (1 1 1)
	format "\tcolor\t{1 1 1 }\n" to:filename
	format "\tmesh_data_file\t{ % }\n\t}\n" (file_pfx+"-"+mesh_obj.name+".sm") to:filename
	format "CREATE\t{ %\n\t}\n" (file_pfx+"-"+mesh_obj.name) to:filename
)

fn writeTexbodyUpdate mesh_obj k filename =
(
	format "UPDATE_GEOM\t{ %\n" (file_pfx+"-"+mesh_obj.name) to:filename
	format "\tTEXBODY\t{\n" to:filename
	format "\t\t name\t%\n" (file_pfx+"-"+mesh_obj.name) to:filename
	format "\t\t xform\t{" to:filename
	
	format "{% % % % }{% % % % }{% % % % }" \
	mesh_obj.transform[1][1] mesh_obj.transform[3][1] -mesh_obj.transform[2][1] mesh_obj.transform[4][1] \
	mesh_obj.transform[1][3] mesh_obj.transform[3][3] -mesh_obj.transform[2][3] mesh_obj.transform[4][3] \
	-mesh_obj.transform[1][2] -mesh_obj.transform[3][2] mesh_obj.transform[2][2] -mesh_obj.transform[4][2] \
	to:filename

	format "{0.0 0.0 0.0 1.0 }}\n" to:filename
	format "\t\t xfdef\t{ DEFINER\n" to:filename
	format "\t\t\tDEFINER\t{\n" to:filename
	format "\t\t\t\tout_mask\t1\n" to:filename
	format "\t\t\t\tinputs\t{ }\n" to:filename
	format "\t\t\t\t} }\n" to:filename
	
	-- COLOR - HACKED TO (1 1 1)
	format "\t\tcolor\t{1 1 1 }\n" to:filename
	
	format "\t\tmesh_data_update_file\t{ % }\n" (file_pfx + "-" + mesh_obj.name + "[" + (leadingZero k 5) + "]" + ".sm") to:filename

	format "\t\t}\n\t}\n" to:filename
)

-- ###################################
-- MAIN PART OF SCRIPT

for fctr=1 to nframes do
(
	sliderTime=startFrame+fctr-1
	file3 = createfile (fname + "[" + (leadingZero (fctr+startFrame) 5) + "].jot")
	format "#jot\n\n" to:file3
	for ctr=1 to (mctr-1) do
	(
		mesh=copy mesh_objs[ctr]
		convertToMesh mesh
		mesh.name=mesh_objs[ctr].name
		writeTexbodyUpdate mesh (fctr+startFrame) file3
		writeCamera cam_objs[1] file3
		if fctr == 1 then
		(
			writeTexbody mesh f
			mfile = createfile (fname + "-" + mesh.name + ".sm")
			writeMesh mesh mfile
			close mfile
			mfile2 = createfile (fname + "-" + mesh.name + "[" + (leadingZero (fctr+startFrame) 5) + "].sm")
			writeMeshVerts mesh mfile2
			close mfile2
		)
		else
		(
			mfile2 = createfile (fname + "-" + mesh.name + "[" + (leadingZero (fctr+startFrame) 5) + "].sm")
			writeMeshVerts mesh mfile2
			close mfile2
		)
		delete mesh
	)
	close file3
)

writeCamera cam_objs[1] f
writeWindow f
writeView f

close f
messagebox("Exporting Jot files completed.")